#include "common.h"
#include "const.h"
#include <iostream>
#include <fstream>
#include <cmath>
#include <time.h>

using namespace std;

#include <cstring>
#include <cstdlib>

double Get_distance(LOCATION , LOCATION );

/*
/*===================================================================
FUNCTION: UE::Channel_Update_MIMO( void )



===================================================================*/
void MS::Channel_Update_MIMO(int ms_idx)
{
	_self_idx = ms_idx;

	/*===================================================================
	FUNCTION : Declare_Ch_matrix()
	OUTPUT   : H_m, HI_m(interference channel)
	===================================================================*/
	if (CH_CAL == 1)
	{
		Declare_ch_matrix();
	}
	/*===================================================================
	FUNCTION : Fourier_Transform_of_Channel(_self_idx)
	OUTPUT   : H_m, HI_m(interference channel)
	===================================================================*/
	Fourier_Transform_of_Channel(_self_idx);

	/*===================================================================
	FUNCTION : Quantization_of_Ch()
	INPUT    : H_m
	OUTPUT   : PMI(cw_idx)
	===================================================================*/
	Quantization_of_Ch();

	/*===================================================================
	FUNCTION : CQI_Update()
	INPUT    : H_m, UE_PATH_LOSS, UE_ICI
	OUTPUT   : CQI(esinr)
	===================================================================*/
	CQI_Update();


	

}

void MS::Quantization_of_Ch(void)
{
	if (CH_CAL == 1) //// run time calc
	{
		for (int rbs_idx = 0; rbs_idx < num_rbs_in_1ms; rbs_idx++)
		{
			JacobiSVD <MatrixXcd> svd(H_m[rbs_idx], ComputeFullU | ComputeFullV);    // H = RxT, V = TxT, U = RxR  

			MatrixXcd SVD_V;
			MatrixXcd SVD_V_hermitian;
			VectorXcd H_eq;
			RowVectorXcd H_eq_second;

			SVD_V = svd.matrixV();
			SVD_V_hermitian = SVD_V.adjoint();

			H_eq_second = H_m[rbs_idx].row(0);

			//H_eq = SVD_V_hermitian.col(0);
			H_eq = SVD_V.col(0);

			//PMI_vector[rbs_idx][t%cqi_history_length] = H_eq.conjugate();
			PMI_vector[rbs_idx][t%cqi_history_length] = H_eq_second.transpose();       
			
			PMI[rbs_idx][t%cqi_history_length] = 0;  // No use


		}
	}
}





void MS::CQI_Update(void)
{

	if (CH_CAL == 1)  //// run-time calculate
	{
		//if (NUM_TX != 4)   // No Use PMI
		if (Scheduling_Type == 0)  // round robin scheduling
		{
			for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
			{
				double sum_of_ch_coef = 0.;
				//complex<double> s(1, 2);
				for (int r_idx = 0; r_idx < NUM_RX_Port; r_idx++) // Received    Channel
				{
					for (int t_idx = 0; t_idx < NUM_TX_Port; t_idx++) // Transmitted Channel
					{
						sum_of_ch_coef += pow(abs(H_m[rb_idx](r_idx, t_idx)), 2.);
					}
				}

				double geometry = link[_self_idx].str_signal - link[_self_idx].interference; // (dB scale) // == (link[_self_idx].UE_PATH_LOSS / link[_self_idx].UE_ICI) (linear scale)

				double linear_signal = dBm2linear(link[_self_idx].str_signal);
				double linear_interference = dBm2linear(link[_self_idx].interference);

				linear_interference = linear_interference + noise;
				
				CQI[rb_idx][abs(t) % cqi_history_length] = sum_of_ch_coef * (linear_signal / linear_interference); /// 

				//cout << "CQI = " << CQI[rb_idx][abs(t) % cqi_history_length] << endl;
				//getchar();
			}

		}
		else if (Scheduling_Type == 1)  //MU MIMO scheduling..
		{
			for (int rbs_idx = 0; rbs_idx < num_rbs_in_1ms; rbs_idx++)
			{
				MatrixXcd M, A, B, C;
				VectorXcd w;
				RowVectorXcd u;
				MatrixXcd eye_R = MatrixXcd::Identity(NUM_RX_Port, NUM_RX_Port);
				MatrixXcd eye_T = MatrixXcd::Identity(NUM_TX_Port, NUM_TX_Port);


				double sinr_estimate;
				double linear_signal = pow(10., link[_self_idx].str_signal / 10.);
				double linear_interference = pow(10., link[_self_idx].interference / 10.);

				linear_interference = linear_interference + noise;

				MatrixXcd H_eq = sqrt(linear_signal) * H_m[rbs_idx];  // R x T


				
				w = PMI_vector[rbs_idx][t%cqi_history_length].conjugate();   // Tx1 
				//w = PMI_vector[rbs_idx][t%cqi_history_length];   // Tx1    
				A = H_eq* w * w.adjoint() * H_eq.adjoint();    // RxT * Tx1 *  1xT * TxR = RxR
				B = H_eq * (eye_T - w * w.adjoint()) * H_eq.adjoint();   //// RxT * ( TxT - Tx1 * 1xT ) * TxR = RxR
				C = linear_interference * eye_R;  /// RxR

				//    u = inv( C + B ) * H_eq * w;
				//    u = u.H();

				M = (B + C).inverse() * A;

				JacobiSVD <MatrixXcd> svd(M, ComputeFullU | ComputeFullV);    // H = RxT, V = TxT, U = RxR 

				MatrixXcd SVD_U;
				MatrixXcd SVD_U_hermitian;

				SVD_U = svd.matrixU();
				SVD_U_hermitian = SVD_U.adjoint();

				u = SVD_U_hermitian.row(0);

				sinr_estimate = (u * A * u.adjoint()).norm() / (u * (B + C) * u.adjoint()).norm();


				CQI[rbs_idx][abs(t) % cqi_history_length] = sinr_estimate;
			}
		}

	}
}



double MS::GetSpectralEfficiency(double sinr_estimate, int &MCS)
{
	///


	if (sinr_estimate < -7.8474) { MCS = -1; return 0.0; }
	else if (sinr_estimate >= -7.8474 && sinr_estimate < -6.2369) { MCS = 0; return  0.1523; }
	else if (sinr_estimate >= -6.2369 && sinr_estimate < -4.3591) { MCS = 1; return 0.2344; }
	else if (sinr_estimate >= -4.3591 && sinr_estimate < -1.9319) { MCS = 2; return 0.3770; }
	else if (sinr_estimate >= -1.9319 && sinr_estimate < 0.1509) { MCS = 3; return 0.6016; }
	else if (sinr_estimate >= 0.1509 && sinr_estimate < 1.9976) { MCS = 4; return 0.8770; }
	else if (sinr_estimate >= 1.9976 && sinr_estimate < 4.7278) { MCS = 5; return 1.1758; }
	else if (sinr_estimate >= 4.7278 && sinr_estimate < 6.2231) { MCS = 6; return 1.4766; }
	else if (sinr_estimate >= 6.2231 && sinr_estimate < 8.0591) { MCS = 7; return 1.9141; }
	else if (sinr_estimate >= 8.0591 && sinr_estimate < 9.8585) { MCS = 8; return 2.4063; }
	else if (sinr_estimate >= 9.8585 && sinr_estimate < 11.8432) { MCS = 9; return 2.7305; }
	else if (sinr_estimate >= 11.8432 && sinr_estimate < 13.4893) { MCS = 10; return 3.3223; }
	else if (sinr_estimate >= 13.4893 && sinr_estimate < 15.3598) { MCS = 11; return 3.9023; }
	else if (sinr_estimate >= 15.3598 && sinr_estimate < 17.4435) { MCS = 12; return 4.5234; }
	else if (sinr_estimate >= 17.4435 && sinr_estimate < 19.2155) { MCS = 13; return 5.1152; }
	else if (sinr_estimate >= 19.2155) { MCS = 14; return 5.5547; }
	else
	{
		MCS = -1; return -1;
	}

	
}

void MS::Received_SINR(void)
{
	double noise = pow(10, (-174.0 / 10.0)) * (bandwidth);  // bandwidth = Hz
	MatrixXcd tempRI, tempRH, tempRHr, tempM, tempIRC, temph, eye, signal, interferencePlusNoise;
	tempRI = MatrixXcd::Zero(NUM_RX_Port, NUM_RX_Port);
	tempRHr = MatrixXcd::Zero(NUM_RX_Port, NUM_RX_Port);
	tempRH = MatrixXcd::Zero(NUM_TX_Port, NUM_TX_Port);
	eye = MatrixXcd::Identity(NUM_RX_Port, NUM_RX_Port);

	VectorXd FrequencySINR(num_rbs_in_1ms);

	// For SVD
	MatrixXcd tempV;


	for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
	{
		tempRHr = tempRHr + (H_m[rb_idx] * H_m[rb_idx].transpose()) / num_rbs_in_1ms;   /// tempRHr = num_RX x num_RX
		tempRH = tempRH + (H_m[rb_idx].transpose() * H_m[rb_idx]) / num_rbs_in_1ms;   /// tempRH  = num_TX x num_TX
	}

	JacobiSVD <MatrixXcd> svd(tempRH, ComputeFullU | ComputeFullV);    ////// U = RX x RX, V = TX x TX



	tempV = svd.matrixV();

	digitalPrecodingMatrix = tempV.col(0);


	for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
	{

		tempM = H_m[rb_idx] * tempV.col(0);
		//tempIRC = tempM.transpose() * (tempM*tempM.transpose() + noise*eye + (0.1*randnum.u())*eye).inverse();
		tempIRC = tempM.transpose() * (tempM*tempM.transpose() + (0.1*randnum.u())*eye).inverse();
		temph = tempIRC*H_m[rb_idx];


		signal = temph*temph.transpose();



		//interferencePlusNoise = tempIRC*(noise*eye)*tempIRC.transpose();

		interferencePlusNoise = tempIRC*tempIRC.transpose();   ////



		FrequencySINR(rb_idx) = abs(real(signal(0, 0)) / real(interferencePlusNoise(0, 0)));


	}

	double SUM = 0.;

	for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
	{
		SUM += log2(1.0 + FrequencySINR(rb_idx));


	}

	downlinkESINR = pow(2., SUM / num_rbs_in_1ms) - 1. + downlinkHARQeSINR;
	downlinkESINRdB = 10.0 * log10(downlinkESINR);


}




void MS::Fourier_Transform_of_Channel(int ms_idx)
{
	if (CH_CAL == 1)   //// runtime calc
	{
		int self_idx = ms_idx;
		int A = 0;

		if (TYPE == 11 && num_Indoor_TRxP == 1)
		{
			A = link[self_idx]._sector_in_control;
		}
		else
		{
			A = (int)(link[self_idx].adj_sector[0] / 3);
		}

		for (int r_idx = 0; r_idx < NUM_RX_Port; r_idx++) // Received    Channel
		{
			for (int t_idx = 0; t_idx < NUM_TX_Port; t_idx++) // Transmitted Channel
			{
				int pointer = (int(DFT_SIZE) - (num_rbs_in_1ms*num_freq_per_rbs)) / 2;   ////  (1024 - 50*12) / 2 = 212
				double sum_for_normal = 0;
				for (int rbs_idx = 0; rbs_idx < num_rbs_in_1ms; rbs_idx++) // Subcarrier idx
				{
					int freq_idx = (pointer + (num_freq_per_rbs / 2) + (rbs_idx*num_freq_per_rbs)) * subcarrier_spacing; //// 15000 = 15kHz subcarrier   ///// (212 + (12/2) + (rbs_idx * 12)) * 15k
					for (int path_idx = 0; path_idx < channel[A][self_idx].NUM_PATH_for_channelcoeff; path_idx++)
					{
						complex <double> dft_exp(0.0, -2 * pi*freq_idx*(channel[A][self_idx].delay[path_idx]));
						H_m[rbs_idx](r_idx, t_idx) += channel[A][self_idx].CHIR[ (int)(link[self_idx].adj_sector[0]%3) ][t_idx][r_idx][path_idx] * exp(dft_exp);

					}
					sum_for_normal += pow(abs(H_m[rbs_idx](r_idx, t_idx)), 2.);
				}

				sum_for_normal = sum_for_normal / (double)num_rbs_in_1ms;

				for (int rbs_idx = 0; rbs_idx < num_rbs_in_1ms; rbs_idx++) // Subcarrier idx
				{
					H_m[rbs_idx](r_idx, t_idx) = H_m[rbs_idx](r_idx, t_idx) / sqrt(sum_for_normal);
				}
			}
		}
	}
}






void MS::Declare_ch_matrix(void)
{


	if (H_m == NULL)
	{
		H_m = new MatrixXcd[num_rbs_in_1ms];
	}



	for (int freq_idx = 0; freq_idx < num_rbs_in_1ms; freq_idx++)
	{
		MatrixXcd temp(NUM_RX_Port, NUM_TX_Port);

		for (int r_idx = 0; r_idx < NUM_RX_Port; r_idx++)
		{
			for (int t_idx = 0; t_idx < NUM_TX_Port; t_idx++)
			{
				temp(r_idx, t_idx) = 0.;
			}
		}
		H_m[freq_idx] = temp;
	}



}

