
/*
////////////////////////
5G DownLink System Level Simulator
////////////////////////
*/

#include <stdio.h>
#include <time.h>
#include "common.h"

void Set_simul_param(int argc, char *argv[]);
void Set_Parameter(int scenario);
void StandardInitialization();
void InitializeSystem();
void Set_fastfading_param();
void Allocate_memory(void);
void Initialdrop();
void Initialize_CHANNEL();
void Print_Location(int drop_idx);
void Parameter_initialization();
void Generate_SSP();
void Compute_Channel_Coef();
void Update_Channel_Coef();
void ClearMAPs2StartADrop();
void PerDropStatistics(int);
void Measure();
void Generate_LSP(void);
void Link_configuration();
void Get_CouplingLoss();
void Coupling_Loss_CDF(void);
void Geometry_CDF(void);
void Delete_memory(void);
void Delete_CHIR_memory(void);
void loading(void);
void GetCapacity(void);

int main(int argc, char *argv[])
{

	CH_CAL = 1;                  //// for runtime calculate
	//Scheduling_Type = 0;         //// roundrobin scheduling
	Scheduling_Type = 1;         //// MU MIMO scheduling
	Calibration_mode = 0;        //// 1= Calibration mode, 0 = Full simulation 

	Set_simul_param(argc, argv);
	Set_Parameter(scenario);
	StandardInitialization(); //Table Setting
	InitializeSystem(); 
	Set_fastfading_param();
	Allocate_memory();

	for (drop_idx = 0; drop_idx < num_drop; drop_idx++)
	{
		cout << "Drop # " << drop_idx << "/" << num_drop << " ing... " << endl;
		cout << "  " << endl;

		Initialdrop();
		cout << "initialdrop DONE" << endl;
		cout << "  " << endl;

		Initialize_CHANNEL();
		Generate_LSP();
		Generate_SSP();
		Link_configuration();

		if (Calibration_mode == 0)  /// normal simulation
		{
			ClearMAPs2StartADrop();
		}

		Get_CouplingLoss();
		Print_Location(drop_idx);

		if (Calibration_mode == 0)  /// normal simulation
		{
			Parameter_initialization();
			cout << "Parameter initialization done " << endl;
			cout << "  " << endl;

			Compute_Channel_Coef();

			for (t = 0; t < N_pf + run_times + SCHEDULE_DELAY; t++)
			{
				cout << " " << endl;
				cout << "t = " << t << endl;
				
				Update_Channel_Coef();

				if (t > SCHEDULE_DELAY)  // SCHEDULE_DELAY = 4
				{
					for (int idx = 0; idx < num_SECTORS; idx++) ///NUM_SECTORS = 57 
					{
						sector[idx].Scheduling(Scheduling_Type, idx);

					}
					cout << " Scheduling done" << endl;
				}
				cout << " Channel Update START " << endl;

				for (int idx = 0; idx < num_MS; idx++)
				{
					ms[idx].Channel_Update_MIMO(idx);

				}
				cout << " Channel Update DONE " << endl;

				if (t > SCHEDULE_DELAY)
				{
					Measure();
					//GetCapacity();

				}

				//loading();
			}

			PerDropStatistics(drop_idx);
			cout << " .. Delete_CHIR_memory.. " << endl;
			Delete_CHIR_memory();
			cout << " ..DROP finish.. " << endl;

		}
		else  
		{
			///// calibration mode -> only coupling loss, geometry
		}

	}

	Geometry_CDF();
	cout << " check Geometry CDF " << endl;
	Coupling_Loss_CDF();
	cout << " check Coupling Loss CDF " << endl;

	Delete_memory();

	return 0;
}




